:- module(curryast, [varlist/2, split_rule/3, arity/2]).

% We use high-order prolog.
:- use_package(hiord).

:- include(operators).


% split_rule(Rule,Head,Body):-
% Toma una regla y la divide en Cabeza y Cuerpo

split_rule(Rule, Head, Body) :- 
	Rule = (id(_)@Head) := Body.


% Arity of a type.

arity(X,0) :-
	var(X),!.

arity((_->B), X) :-
	!,
	arity(B, X1),
	X is X1+1.

arity(_,0).



% Strange or duplicate predicates

%------------------------------------
% Goal translation.
%------------------------------------

% Belongs is member but doesn't instance vars.
belongs(X, [Y|_]) :-
	X == Y,
	!.

belongs(Y, [_|Xs]) :-
	belongs(Y,Xs).



varlist(G, Vs) :-
	varlist3(G,[],Vs),
	!.

%

varlist3([], Ac, Ac) :-
	!.
varlist3(I@Ps, Ac, Ac1) :-
	varlist3(I, Ac, Ac2),
	varlist3(Ps, Ac2, Ac1).

varlist3(not(A), Ac, Vs) :-
	varlist3(A,Ac,Vs).

% varlist3(num(_),Ac,Ac).
varlist3([id(findall), _, _, var(_)], Ac, Ac) :-
	!.
varlist3([id(bagof), _, _, _, var(_)], Ac, Ac) :-
	!.
varlist3(id(_), Ac, Ac) :- 
	!.
varlist3(str(_), Ac, Ac) :-
	!.
varlist3([H|Ts], Ac, Vs) :-
	!,
	varlist3(H, Ac, Ac1),
	varlist3(Ts, Ac1, Vs).

varlist3(var(V), Ac, Ac) :-
	belongs(var(V), Ac),
	!.

varlist3(var(V), Ac, [var(V)|Ac]).

varlist3(Exp, Ac, Ac2) :-
	Exp =.. [_Op, Left, Right],
	!,
	varlist3(Left,  Ac,  Ac1),
	varlist3(Right, Ac1, Ac2).

varlist3(Exp, Ac, Ac2) :- 
	Exp =.. [_Id|R],
	!,
	varlist3(R,Ac,Ac2).



%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

