:- module(main, [main/0], _).

:- use_module(library(lists),   [append/3]).
:- use_module(library(system),  [file_exists/1]).
:- use_module(library(write),   [write/1]).
:- use_module(library(terms),   [atom_concat/2]).
:- use_module(library(dec10_io),[tell/1, told/0] ).


:- use_module(error,      [do_error/1]).
:- use_module(trans_lazy, [dump/0]).
:- use_module(parser,     [parse/1] ).
:- use_module(typer,      [infr_types/0] ).
:- use_module(lambda,     [process_lambdas/0] ).
:- use_module(symtab,     [clean_symtab/0, get_rules/2] ).

% egallego@babel: Sloth main module.
% Initial version: only compiles the file specified.

main :-
	catch(sloth_main, E, do_error(E)).

sloth_main :-
	current_prolog_flag(argv, Args),
	(
	    get_args(file, Args, File) ->
	    check_file(File),
	    make_pl_suffix(File, FileOut),
	    (
		get_args(dump, Args, _) ->
		dump_abstree(File, FileOut)
	    ;
		compila(File, FileOut),
		write('File '), write(File), 
		write(' is compiled to '), write(FileOut), 
		write('\n')
	    )
	;
	    throw(argument_error)
	).

% Debug function, writes symbol table to file.
dump_abstree(In, Out) :-
	clean_symtab,
	parse(In),
	!,
	tell(Out),
	dump_rules,
	told.

% For more pretty printing.
:- include(operators).

dump_rules :-
	get_rules(N, R),
	write(N),
	display(': '),
	write(R),
	display('\n'),
	fail.
dump_rules.	

compila(In,Out):-
	clean_symtab,
	parse(In),
	!,
	process_lambdas,
	!,
	infr_types,
	!,
	tell(Out),
	dump,
	told.

% Checks the file exists.
check_file(File):-
	(
	    file_exists(File) -> 
	    true
	;    
	    atom_concat(['File does not exist: ', File], E),
	    throw(E)
        ).

% Changes a f.curry to f.pl
make_pl_suffix(OldFile, NewFile) :-
	atom_codes(OldFile, OFString),
	append(BaseName, ".curry", OFString),
	append(BaseName, ".pl", NFString),
	atom_codes(NewFile, NFString),!.

make_pl_suffix(_, _) :- !,
	throw('Error, filename must end in .curry').
	
% get_args(property, Args, Res) gets from a list of arguments
% the one requested.

get_args(dump, ArgList, '-v') :-
	member('-v', ArgList).

get_args(file, [File], File).
get_args(file, [_|LS], File) :-
	get_args(file, LS, File).


%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

