:- module(symtab, [clean_symtab/0, add_rule/2, get_rules/2, update_rules/2, set_object_property/3, 
	           get_object_property/3, add_constructor/2, get_constructors/2,
		   delete_property/2, replace_object_property/3]).

% This module implements a function oriented symbol table.

:- use_module(library(dynamic),    [assert/1, retract/1, retractall/1]).

% 

% Private dynamics

:- dynamic(func_rules/2).
:- dynamic(object_properties/3).
:- dynamic(type_cons/2).

clean_symtab :-
	retractall(func_rules(_, _) ),
	retractall(object_properties(_, _, _) ),
	retractall(type_cons(_, _) ).

% New symtab begins here

% Rule handling predicates.
add_rule(Name, AbsTree) :-
	retract(func_rules(Name, L)),
	assert(func_rules(Name, [AbsTree|L])).

% If we don't have any rules, add the first.
add_rule(Name, AbsTree) :-
	assert(func_rules(Name, [AbsTree])).

update_rules(Name, Rules) :-
	retract(func_rules(Name, _)),
	assert(func_rules(Name, Rules)).

get_rules(Name, ListOfAbsTrees) :-
	func_rules(Name, ListOfAbsTrees).

% General property handling.
set_object_property(Name, Property, Value) :-
	check_property(Property),
	assert(object_properties(Name, Property, Value)).

get_object_property(Name, Property, Value) :-
%	check_property(Property),
	object_properties(Name, Property, Value).

delete_property(Name, Property) :-
	retractall(object_properties(Name, Property, _)).

replace_object_property(Name, Property, Value) :-
	delete_property(Name, Property),
	set_object_property(Name, Property, Value).

% Function properties are well defined and symbol
% table enforces this rectriction.
is_property(type).
is_property(arity).
is_property(eval).

% FIXME: provisional until I see if can replace link in the whole Sloth.
is_property(constructs).

% FIXME: Temporal typer properties.
is_property(cycle).
is_property(current_func).
is_property(current_rule).
is_property(change).

% If P is not a property throw and error.
check_property(P) :-
	is_property(P),
	!.

check_property(P) :-
	throw(invalid_property(P)).


% Type database:
% 
% A data type in Curry usually has some constructors.
%
% Data type -> List of Constructors.
% Data type -> arity, etc...
%
% Constructor -> Arity, Type
%
add_constructor(Type, Cons) :-
	retract(type_cons(Type, ConsL)),
	assert(type_cons(Type, [Cons|ConsL])).

add_constructor(Type, Cons) :-
	assert(type_cons(Type, [Cons])).

get_constructors(Type, ListOfCons) :-
	type_cons(Type, ListOfCons).
 
% For constructors us  


%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

