% trans_lazy.pl
% 14 de marzo de 2003
%
:- module(trans_lazy, [dump/0]).

% Module use cleanup.

:- use_module(prelude,       [forbidden/1]).
:- use_module(prolog_writer, [write_rule/1, write_rules/1, write_exp/2] ).
:- use_module(aux,           [qsort/2, term_at/3, give_name/2] ).

:- use_module(deftree,       [defTree/3, call_defTree/2] ).
:- use_module(curryast,      [arity/2, varlist/2, split_rule/3]).

:- use_module(symtab,        [clean_symtab/0, get_rules/2, get_constructors/2, get_object_property/3] ).

%

:- use_module(library(lists),      [append/3, length/2, reverse/2, list_concat/2] ).
:- use_module(library(aggregates), [findall/3] ).

:- include(operators).

% This operator is used only here
:- op(500, xfx, #).


%%%%%%%%%%%%%%%%%%
% This code was in aux and should be removed because has only 1 user.
%%%%%%%%%%%%%%%%%%

is_in_hnf(A) :-
        (
	    var(A)
	;
	    number(A)
	), 
	!.

is_in_hnf(str(_)) :-
	!.
is_in_hnf(char(_)) :-
	!.
is_in_hnf([]) :- 
	!.
is_in_hnf([_|_]) :- 
	!.
is_in_hnf('Valid') :- 
	!.
is_in_hnf(Term) :- 
	functor(Term, C, _),
	constructor(C),
	!.

constructor('Nil').
constructor('Cons').
constructor('True').
constructor('False').
constructor(num).
constructor(',').

%

subst_term(Term, Pos, nil, Res) :-
	!,
	subst_term(Term,Pos,[],Res).

subst_term(Term,Pos,cons(A,B),Res) :-
	!,
	subst_term(Term,Pos,[A|B],Res).

subst_term(_Term,[],Subterm,Subterm) :-
	!.

subst_term(Term, Pos, Subterm, Term2) :-
	functor(Term, Functor, Arity),      
		%Cambia el termino en la posicion indicada
	functor(Term2, Functor, Arity),     % y lo sustituye por Subterm.
	subst_term_aux(Term, Term2, Pos, Subterm, Arity).

subst_term_aux(_, _, _, _, 0) :-
	!.

subst_term_aux(Term, Term2, [N|Ps], Subterm, N) :-
	!,
	arg(N,Term,T),
	subst_term(T,Ps,Subterm,Term3),
	arg(N,Term2,Term3),
	N1 is N-1,
	subst_term_aux(Term,Term2,[N|Ps],Subterm,N1).

subst_term_aux(Term,Term2,[N|Ps],Subterm,N2) :-
	\+ ( N = N2 ),
	arg(N2,Term,T),
	arg(N2,Term2,T),
	N1 is N2-1,
	subst_term_aux(Term,Term2,[N|Ps],Subterm,N1).


% parser2trad(TermPred,TermTrad):- Cambia los subterminos id(A)@Args (Formato
% del parser a A(Args) (formato del traductor). Tambien caambia de id(I) a I

parser2trad( var(X), var(X)) :-
	!.

parser2trad( id([]), []) :- 
	!.

parser2trad(id(':')@[A,As],[B|Bs]) :-
	!,
	parser2trad(A,B),
	parser2trad(As,Bs).

parser2trad(id(P)@Ts,T) :-
	!,
	parser2trad_list(Ts,Ts2),
	T =.. [P|Ts2].

parser2trad(id(P),P) :-
	!.

parser2trad(TermP,TermT) :-
	TermP =.. [Cons|Args],
	Args  =   [_|_],
	!,
	parser2trad_list(Args,ArgsT),
	TermT =.. [Cons|ArgsT].

parser2trad(T,T).

parser2trad_list([], []).

parser2trad_list([T|Ts], [T2|Ts2]) :-
	parser2trad(T,T2),
	parser2trad_list(Ts,Ts2).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
create_n_vars(0, []) :-
	!.
create_n_vars(N, [var(_)|Ts]) :-
	N1 is N-1,
	create_n_vars(N1,Ts).


%%%%%%%%%%%

% Caso 1, la regla empieza con un constructor.
tr_rule(R, Rt) :-
	R = (Head:=Body),
	(
	    ( Body = id(Cons)@_ )
	;
	    ( Body = id(Cons) ) 
	),
	get_object_property(Cons, constructs, _Type),
	!,
	trans_struc_goal(Body, BodyTrad),
	punteros(BodyTrad, BodyTrad2, _Dicc),
	parser2trad(Head, HeadTrad),
	HeadTrad =.. [Pred|Hlist],
	append(Hlist, [BodyTrad2], Hlist2),
	Ht =.. [Pred|Hlist2],
	Rt = prolog(Ht, []).

tr_rule(R,Rt) :-
	R = (Head:=Body),
	parser2trad(Head, HeadTrad),
	HeadTrad =.. [Pred|Hlist],
	append(Hlist, [var(R0)], Hlist2),
	HeadTrad2 =.. [Pred|Hlist2],
	trans_struc_goal(Body, BodyTrad),
	punteros(BodyTrad, BodyTrad_punteros, _Dicc),
	BodyTrad2 = [tohnf(BodyTrad_punteros, var(R0))],
	Rt_intermedio = prolog(HeadTrad2,BodyTrad2),  
	(
	    get_rules(Pred, _Rules),
	    get_object_property(Pred, type, Type),
	    arity(Type,Type_arity),
	    length(Hlist,Prog_arity),
	    Type_arity > Prog_arity ->                    
					% Reglas con aridad inferior a la del tipo (prueba.bab)
	    Dif is Type_arity - Prog_arity,
	    create_n_vars(Dif, Vars),
						%  reverse(Vars, VarsRev),
            append(Hlist, Vars, Hlist3),
	    append(Hlist3, [var(R0)], Hlist4), %Parametro de salida
	    Head3 =.. [Pred|Hlist4],
		% punteros(BodyTrad,Body_punteros,_Dicc),
            Body3 = [tohnf(apply_list(BodyTrad_punteros, Vars)#var(R0), var(R0))],
		% punteros(Body3,Body_punteros,_Dicc),
	    Rt = prolog(Head3,Body3)
	;
	    Rt = Rt_intermedio
	).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Traduccion del objetivo.

% trans_struc_goal: Toma el objetivo dejado por el parser y lo pasa a formato
% aceptable para tohnf/2: and(X,or(Y,Z))...


trans_struc_goal(cond(Constraint,Exp), cond(Ct,Et)) :-
	!,
	trans_struc_goal(Constraint,Ct),
	trans_struc_goal(Exp,Et).

trans_struc_goal(constraint(List), constraint(Listt)) :-
	!,
	trans_struc_goal_list(List,Listt).

trans_struc_goal(var(V)@Pars, apply_list(var(V),Pars2)) :-
	!,
	trans_struc_goal_list(Pars,Pars2).

trans_struc_goal(id(I)@Pars, pap(I,Quedan,Pars2)) :-
	get_rules(I, _Rules),
	get_object_property(I, type, Type),
	arity(Type, Arity),
	length(Pars,NumPars),
	NumPars < Arity,
	!,
	Quedan is Arity - NumPars,
	reverse(Pars, ParsRev),
	trans_struc_goal_list(ParsRev,Pars2).

% trans_struc_goal(id(I), X) :-
% 	get_rules(I, _Rules),
% 	!,
% 	get_object_property(I, arity, Arity),
% 	(
% 	    Arity==0 -> 
% 	    X = I 
% 	; 
% 	    X = pap(I,Arity,[])
% 	).

trans_struc_goal(id(':')@[H,T], [Htsg|Ttsg]):-
	!,
	trans_struc_goal(H,Htsg),
	trans_struc_goal(T,Ttsg).

trans_struc_goal(id(I)@Pars, pap(I,Quedan,Pars2)):- %% Constr. currif.
	get_object_property(I, constructs, _Type),
	get_object_property(I, type, Type),
	arity(Type,Arity),
	length(Pars, NumPars),
	NumPars < Arity,
	!,
	Quedan is Arity - NumPars,
	reverse(Pars, ParsRev),
	trans_struc_goal_list(ParsRev,Pars2).

% trans_struc_goal(id(I), X) :-
% 	get_object_property(I, constructs, _Type),
% 	get_object_property(I, type, Type),
% 	arity(Type,Arity),
% 	(
% 	    Arity == 0 -> 
% 	    X = I ; 
% 	    X = pap(I, Arity, [])
% 	).

trans_struc_goal(id(I)@Pars, T) :- 
	!,
	trans_struc_goal_list(Pars, Parst),
	T =.. [I|Parst].

trans_struc_goal(A@Pars, T) :-
	trans_struc_goal(A,At),
	At =.. [_Pred|_],
	!,
	trans_struc_goal_list(Pars,Pars2),
	T = apply_list(At,Pars2).

trans_struc_goal(E, E2):-
	parser2trad(E,E2). 

  
trans_struc_goal_list([],[]).
trans_struc_goal_list([A|As], [B|Bs]) :-
	trans_struc_goal(A,B),
	trans_struc_goal_list(As,Bs).

trans_struc_choice_list([],[]).
trans_struc_choice_list([(constraint(C)->Exp)|Ls], [(constraint(Ct)->Expt)|Lst]) :-
	trans_struc_goal(constraint(C),constraint(Ct)),
	trans_struc_goal(Exp,Expt),
	trans_struc_choice_list(Ls, Lst).

% Anadir punteros para los resultados de las expresiones

punteros(var(A),var(A),_):-!. % var no es ninguna funcion ni constructor

punteros(P,Pt,Dicc):-  
	P =.. [pap,FunctionName|Rest],
	!, 
		% No poner puntero a la funcion de un pap.
	punteros_list(Rest, Restp, Dicc),
	Pt_aux =.. [pap, FunctionName|Restp],
	member(P#Res, Dicc),
	Pt = (Pt_aux#var(Res)).

punteros(P, Pt, Dicc):-
	P =.. [Cons|Pars],
	(
	    is_in_hnf(P)
	;
	    (
		get_object_property(Cons, constructs, _Type)
	    ;
		Cons = num
	    )
	),
	!, 
					% o Pars=[]?
	punteros_list(Pars,Parst,Dicc),
	Pt =.. [Cons|Parst].

punteros(P,Pt,Dicc):-
	P =.. [NoCons|Pars],
	punteros_list(Pars, Parst, Dicc),
	member(P#Res,Dicc),
	P2 =.. [NoCons|Parst],
	Pt = (P2#var(Res)).

punteros_list([],[],_Dicc).
punteros_list([A|As],[B|Bs],Dicc):-
	punteros(A, B, Dicc),
	punteros_list(As, Bs, Dicc).
  
trans_exp_lazy(G,prolog(H,B)):-
	trans_struc_goal(G,Gt),
	varlist(G, Vars),
	reverse(Vars, Varsrev),
	append(Varsrev,[var('R0')], Vars2),
	H =.. [goal|Vars2],
	punteros(Gt, Gt2, _Dicc),
	varlist(Gt2, Punteros),
	give_name(Punteros, 1),
	B = tonf(Gt2, var('R0')).


letra_nivel(Pred,Nivel,PredNivel):-
	name(Pred,PredList),
	(Nivel=0->Letra=[];Letra_aux is 96+Nivel,Letra=[Letra_aux]),
	append(PredList,Letra,PredList2),
	name(PredNivel,PredList2).


% Esto es un takeN
comienzos(0,[]):-!.
comienzos(N,[[N]|R]):-
	N>0,N1 is N-1,
	comienzos(N1,R).



tohnf_entry(Pred, Deftree, NumLetra, Rule):- 
	%Genera la regla de tohnf correspondiente a Pred.
	Head=..[tohnf,TermPred#Res,Res],  %Cabeza de la regla resultante.
%	lookup(Pred,defType,Type),
%	arity(Type,Arity),
	get_object_property(Pred, arity, Arity),
	create_n_vars(Arity,Vars),
	TermPred =.. [Pred|Vars],
	(
	    Deftree = branch(Pos,_)-_ ->
	    (
		Rule = [prolog(Head, Body)],
		term_at(TermPred,Pos, var(Var)),
		Body = [tohnf(var(Var), var(Free1)), TermResult],
		append(Vars, [var(Result)], Vars2),
		name(Pred,PredStr),
		(
		    NumLetra = 0 ->
		    Sufijo=[49]
		;
		    (
			Letra is 96 + NumLetra,
			Sufijo = [49,Letra]
		    )
		),
		append(PredStr, Sufijo, PredStr1),
		name(Pred1, PredStr1),
		TermResult_aux =.. [Pred1|Vars2],
		subst_term(TermResult_aux, Pos, var(Free1), TermResult),
		Res = var(Result)
	    )
	;
	    (
		Deftree = regla-_ ->
		name(Pred, PredStr),
		(
		    NumLetra = 0 ->
		    Sufijo = [49]
		;
		    (
			Letra is 96 + NumLetra,
			Sufijo = [49,Letra]
		    )
		),
		append(PredStr, Sufijo, PredStr1),
		name(  Pred1, PredStr1),
		Res   = var(Result),
		append(Vars, [var(Result)], Vars2),
		BodyR =.. [Pred1|Vars2],
		Rule  = [prolog(Head,BodyR)] 
	    ;
		(
		    Deftree = or(Deftrees)-_ ->
		    map_or_trees(Pred,Deftrees,0,Rule) 
		;
		    Rule = [prolog(Head,[id(fail)])]
		)
	    )
	).

% Predicado que traduce un definitional tree (solo branch) porque los otros
% casos ya los hace tohnf_entry.

trans_deftree(Pred,Reglas):-
	call_defTree(Pred, Deftree),
	Deftree = or(Arboles)-_,
	!,
	get_object_property(Pred, type, Type),
	arity(Type,Arity),
	create_n_vars(Arity,Vars),
	T_Acum=..[Pred|Vars],
	map_trans_deftree_aux(Pred,1,T_Acum,Arboles,0,ReglasAux),
	list_concat(ReglasAux,Reglas).

trans_deftree(Pred,Reglas):-
						%  lookup(Pred,defType,Type),
						%  arity(Type,Arity),
	get_object_property(Pred, arity, Arity),
	get_rules(Pred, ReglasPred),
	comienzos(Arity,Comienzos),
	create_n_vars(Arity,Vars),
	TermAcum=..[Pred|Vars],
	defTree(ReglasPred,Comienzos,DTree),
	(
	    DTree = branch(Pos,Arbol)-_Reglas ->
	    trans_deftree_aux(Pred,1,TermAcum,(Pos,Arbol),0,Reglas) 
	;
	    (
		DTree = regla-[N] ->
		trans_deftree_aux(Pred,1,TermAcum,regla-[N],0,Reglas)
	    ;
		fail
	    )
	).  

% trans_deftree_aux(Predicado,Nivel,Termino_acum,Arbol,NumArbol,Reglas).

trans_deftree_aux(_Predicado, _Nivel, _T_Acum, fallo-[], _, []) :-
	!.
trans_deftree_aux(Pred,Nivel,T_Acum,(or(Arboles)-_), _NumArbol, Reglas) :-
	!,
	map_trans_deftree_aux(Pred,Nivel, T_Acum,Arboles, 0, Reglas2),
	list_concat(Reglas2,Reglas).
  
trans_deftree_aux(Predicado, Nivel, _T_Acum, regla-[N], NumArbol, [prolog(HeadR,BodyR)-[N]]) :-
	!,
	name(Predicado,PredStr),
	Na is 48 + Nivel,
	NumStr = [Na],
	append(PredStr,NumStr,PredNStr),
	name(PredN,PredNStr),
	N = Reg,
	tr_rule(Reg,prolog(Head2,BodyR)),
	Head2 =.. [_Pred|Pars],
	letra_nivel(PredN,NumArbol,PredNb),
	HeadR =.. [PredNb|Pars].

trans_deftree_aux(Pred, Nivel, T_Acum, branch(Pos,Ramas)-Lr,NumArbol, [prolog(Head,Body)-Lr|Rs]) :-
	term_at(T_Acum,Pos,T1),
	T_Acum =.. [_|T_AcumList],
	append(T_AcumList,[var(Free)],HeadPars),
	name(Pred,PredStr),
	Na is 48 + Nivel,
	NumStr = [Na],
	Na1 is Na+1,
	NumStr1=[Na1],
	append(PredStr,NumStr1,PredN1Str),
	append(PredStr,NumStr,PredNStr),
	name(PredN,PredNStr),
	name(PredN1,PredN1Str),
	letra_nivel(PredN,NumArbol,PredNb),
	Head =.. [PredNb|HeadPars], %A partir de ahora, generar predN+1(...)
	subst_term(T_Acum,Pos,var(Free2),T_Acum2),
	T_Acum2 =.. [_|T_Acum2List],
	append(T_Acum2List,[var(Free)],Head2Pars),
	letra_nivel(PredN1,NumArbol,PredN1b),
	Body = [tohnf(T1,var(Free2)),TermPred1],
	TermPred1 =.. [PredN1b|Head2Pars],
	N1 is Nivel + 1,
	trans_deftree_aux(Pred,N1,T_Acum,(Pos,Ramas),NumArbol,Rs),
	!.

trans_deftree_aux(Pred,Nivel,T_Acum,(Pos,[(Cons,Tree)|Ts]),NumArbol,Rs) :- !,
	(
	    number(Cons)->
	    ArityCons = 0 
	; 
	    get_object_property(Cons, arity, ArityCons)
	), 
	create_n_vars(ArityCons, Vars),
	TermCons =.. [Cons|Vars],
	subst_term(T_Acum, Pos, TermCons, T_Acum2),
	trans_deftree_aux(Pred, Nivel, T_Acum2, Tree, NumArbol, Rs1),
	trans_deftree_aux(Pred, Nivel, T_Acum, (Pos, Ts), NumArbol, Rs2),
	append(Rs1,Rs2,Rs3),
	Rs = Rs3.

trans_deftree_aux(_,_,_,(_,[]),_,[]).
	
%

get_all_funcs(Functions) :-
	findall(Name, (get_rules(Name, _), \+forbidden(Name)), Functions).

tohnf_entries(Reglas):-
	get_all_funcs(Functions),
	map_tohnf_entry(Functions, Reglas). 

map_tohnf_entry([],[]).
map_tohnf_entry([F|Fs],Rs):-
	call_defTree(F, Tree),
	true, 
	Rs3=[],
	tohnf_entry(F,Tree,0,R),
	append(R,Rs3,Rs3b),
	map_tohnf_entry(Fs,Rs2),
	append(Rs3b,Rs2,Rs).

trans_deftrees(Reglas):-
	get_all_funcs(Functions),
	map_trans_deftree(Functions,Rs), 
	list_concat(Rs,Reglas2),
	qsort(Reglas2,Reglas).

map_trans_deftree([],[]).
map_trans_deftree([F|Fs],[Lr|Lrs]):-
	(
	    trans_deftree(F,Lr) ->
	    true
	;
	    Lr = []
	),
	map_trans_deftree(Fs,Lrs).

map_trans_deftree_aux(_,_,_,[],_,[]).
map_trans_deftree_aux(Pred,Nivel,T_Acum,[Tree|Trees],NumArbol,Reglas):-
	NumArbol1 is NumArbol + 1,
	trans_deftree_aux(Pred, Nivel, T_Acum, Tree, NumArbol, ReglasTree),
	map_trans_deftree_aux(Pred, Nivel, T_Acum, Trees, NumArbol1, ReglasTrees),
	append([ReglasTree], ReglasTrees, Reglas).


% map_or_trees(Pred,Arboles,Num_Rama,Reglas): Aplica tohnf_entry a cada una de las
% alternativas de un arbol or y devuelve el resultado en reglas (una lista de
% ellas).

map_or_trees(_, [], _, []).
map_or_trees(Pred, [T|Ts], N, Z) :-
	N1 is N+1,
	tohnf_entry(Pred, T, N, R),
	map_or_trees(Pred, Ts, N1, Rs),
	append(R, Rs, Z).

% Crea las reglas correspondientes a los constructores en hnf
constructor_rule_list_hnf(Rules) :-
	get_cons_arity_list(ConsMembers),
	tohnf_cons_entries(ConsMembers, Rules).

% Crea las reglas correspondientes a los constructores en nf
constructor_rule_list_nf(Rules):-
	get_cons_arity_list(ConsMembers),
	tonf_cons_entries(ConsMembers, Rules).

filter_cons_for([], []).
filter_cons_for([L|LS], RS) :-
	cons_forbidden(L),
	!,
	filter_cons_for(LS, RS).
filter_cons_for([L|LS], [L|RS]) :-
	filter_cons_for(LS, RS).

cons_forbidden(X) :-
	member(X, ['True','False',num,[],'.',':',',']).

get_cons_arity_list(List) :-
	findall(ConsList, get_constructors(_, ConsList), CCList),
	list_concat(CCList, CList),
	filter_cons_for(CList, CList1),
	map_arity(CList1, List).

map_arity([], []).
map_arity([L|LS], [L-A|RS]) :-
	get_object_property(L, arity, A),
	map_arity(LS, RS).

% Genera reglas de tipo:
% tohnf(kons(H,T),kons(H,T)).
% Creo q bien.

tohnf_cons_entries([], []).

	% regla para constructores de aridad 0
	% Cons-0 es un  par constructor aridad
tohnf_cons_entries([Cons-0|Cs], [R|Rs]) :-
	!,
	R = prolog(tohnf(Cons,Cons),[]),
	tohnf_cons_entries(Cs,Rs).
	% regla para constructores de aridad >0

tohnf_cons_entries([Cons-Arity|Cs], [R|Rs]) :-
	Arity > 0,
	create_n_vars(Arity,Vars),
	Term =.. [Cons|Vars],
	Head = tohnf(Term,Term), %Crea la cabecera con las variables repetidas
	% Genera la tira de tohnf de la regla anterior
	R = prolog(Head, [!] ), %el cuerpo de la regla es vacia
	tohnf_cons_entries(Cs,Rs). 



tohnfs([],[],[]).
tohnfs([Var|Vars],[VarTrad|VarTrads],[tohnf(Var,VarTrad)|Tohnfs]) :-
	tohnfs(Vars,VarTrads,Tohnfs).


% Regla para tonf poner en forma normal las reglas de los constructores.
% Genera reglas de tipo:
% tonf('Zero','Zero').
% tonf('Succ'(R1),'Succ'(R0)):-
%	tonf(R1,R0).
% Creo q bien.


tonf_cons_entries([],[]).

tonf_cons_entries([Cons-0|Cs],[R|Rs]):-
	!,
	R=prolog(tonf(Cons, Cons), []),
	tonf_cons_entries(Cs, Rs).

tonf_cons_entries([Cons-Arity|Cs], [R|Rs]) :-
	Arity > 0,
	create_n_vars(Arity, Vars),
	create_n_vars(Arity, VarsTrad),
	Term     =.. [Cons|Vars],
	Termtrad =.. [Cons|VarsTrad],
	Head     =   tonf(Term,Termtrad),  
	tonfs(Vars,VarsTrad,Tonfs), 
	% Genera la tira de tonf de la regla anterior
        R = prolog(Head, [! | Tonfs]),
	tonf_cons_entries(Cs, Rs).

tonfs([],[],[]).
tonfs([Var|Vars],[VarTrad|VarTrads],[tonf(Var,VarTrad)|Tonfs]) :-
	tonfs(Vars, VarTrads, Tonfs).

% Reglas de paso a nf:

% c_rules_nf: Toma las reglas resultado de c_rules y genera lo mismo con ! y
% con la cabeza cambiada adecuadamente.

%c_rules_nf([],[prolog(tonf(var('E'),var('Enf')),
% [tohnf(var('E'),var('Ehnf')),tonf(var('Ehnf'),var('Enf'))])]).

c_rules_nf([],[]).
c_rules_nf([prolog(Head,Body)|Rs],[prolog(Headt,[!|Bodyt])|Rst]) :-
	Head  =.. [tohnf, A, C],
	Headt =.. [tonf,  A, C],
	c_rules_nf_body(Body, Bodyt),
	c_rules_nf(Rs, Rst).
  

c_rules_nf_body([],[]).
c_rules_nf_body([tohnf(A,B)|R],[tonf(A,B)|Rt]) :-
	c_rules_nf_body(R,Rt).
  
is_in_hnf_constructors :-
	get_constructors(_ , ListOfCons),
	hnf_cons(ListOfCons),
	fail.

is_in_hnf_constructors.

hnf_cons([]).
hnf_cons([L|LS]) :-
	\+member(L,[nil,cons,true,false,num,'.','True','False',[],:,',']),
	write('constructor('),write_exp(L,nonl),write(').\n'),
	hnf_cons(LS).


%%%%%%%%%%%%%%%%%%%% Traduccion del objetivo %%%%%%%%%%%%%%%%%%%%%%%

change_dicc([],[]).
change_dicc([Nombre-_|R],[Nombre is var(Nombre)|Rt]):-
	change_dicc(R,Rt).


%trans_goal_lazy(prolog(goal,exec_batch(B,Dicc2))):- Cabecera anterior

trans_goal_lazy(prolog(goal,[B|Dicc2])):-
	get_rules('@Objective', [G]),
	symtab2pro(G,_,Dicc), %Hallar las variables de programa.
                        % Solo nos interesa la lista de var nombre-v. libre
                        % en Dicc. Solo nos falta cambiar las var. libres por
                        % var(nombre) para que write_exp las escriba con la V
	change_dicc(Dicc,Dicc2),
	trans_exp_lazy(G,prolog(_,B)).
	




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
headers:-
	write(':-multifile constructor/1,tohnf/2,tonf/2,eq/2.'),
	nl,
	write(':-use_module(library(system)).'),
	nl,
	write(':-use_module(library(lists)).'),
	nl,
	write(':-include(nfhnf).'),
	nl.

dump:-
	headers,
	nl,
	is_in_hnf_constructors,
	nl,
	constructor_rule_list_hnf(Rs),  % crea las reglas para constructores en hnf
	write_rules(Rs),
	nl,
	constructor_rule_list_nf(Rnfs), % crea las reglas para constructores en nf
	write_rules(Rnfs),
	nl,
	tohnf_entries(R),
	write_rules(R),
	nl,
	trans_deftrees(Reglas),
	write_rules(Reglas),
	nl,
	(
	    trans_goal_lazy(Goal) ->
	    write_rule(Goal)
	;
	    (
		write('Aviso: No hay objetivo.'),
		nl,
		write('goal.'),
		nl
	    )
	).


% Cdigo traido de parser que slo se usa aqu.

symtab2pro(num(N),N,_) :-
	!.

symtab2pro(var(V),V,_) :-
	var(V),
	!.

symtab2pro(var(VarTerm), V, Env) :-
	!,
	member(VarTerm-V, Env).

%symtab2pro(Term,TermPro,Env):-
%  Term=..[Op|Args],!,
%  name(Op,OpList),
%  OpList=[L|Ls], % Poner los constructores con la 1a minuscula
%  (L>=0'A,L=<0'Z -> % (Para que no sean variables de prolog)
%   L2 is L+32,OpList2=[L2|Ls] ; OpList2=OpList),
%  name(Op2,OpList2),
%  symtab2pro_list(Args,ArgsPro,Env),
%  TermPro=..[Op2|ArgsPro].

symtab2pro(Term, TermPro, Env) :-
	Term =.. [Op|Args],
	!,
        symtab2pro_list(Args, ArgsPro, Env),
	TermPro=..[Op|ArgsPro].
						% Poner los constructores entre comillas
						% (Para que no sean variables de prolog


symtab2pro(Term, Term, _).

symtab2pro_list([], [], _).
symtab2pro_list([T|Ts], [Tpro|Tspro], Env) :-
	symtab2pro(T, Tpro, Env),
	symtab2pro_list(Ts, Tspro, Env).

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

